/* All functions which deal with Philips 7111 decoder chip */

#include <linux/delay.h>
#include "igs.h"
#include "video.h"

/*for 7111 initialization*/
#define P7111_Count 17
static unsigned char Phil7111_02[P7111_Count]=
			{/* 2    3    4    5    6    7    8    9 */
				0xc0,0x23,0x00,0x00,0xeb,0xe0,0x48,0x40,
			 /* a    b    c    d    e    f   10   11 */
				0x80,0x47,0x40,0x00,0x02,0x00,0xc0,0x0c,
			 /*12 */
				0x80
			}; /*reg_00 to reg_12*/

/**********************************************************
   Detect which decoder is hooked up
   in: None
   return
        PHILIPS7111 - Philips 7111 Decoder
        NODECODER   - No decoder detected.
*********************************************************/
int  DetectDecoder(void)
{
  int wDecoderID = NODECODER;

  if (TRUE == sendOneByte(P7111Port, 0x02, 0xD8))
  {
//printk("Using Philips 7111 Decoder.\n");
    wDecoderID								= PHILIPS7111;
    VDev.gnWhichDecoder				= wDecoderID;
    VDev.gusHighResThreshold	= 352*2;  /* indicator: Width < 352 or not */
  }

  return wDecoderID;
}

/**********************************************************
 Initialize Philips 7111
 in:   none
 return  TRUE  if successful
   FALSE if error
 *********************************************************/
int InitPhil7111(void)
{
  if (sendMoreByte(P7111Port, 0x02, Phil7111_02, P7111_Count) == FALSE)
    return FALSE;

  return TRUE;
}

/**
 * InteGraphics Systems code:
 * Communication routines to talk to the Philips SAA7111 via the CyberPro2000
 * registers 3ce (index:b6) and 3cf (data).
 * Communication between CyberPro2000 and SAA7111 is accomplished by using the
 * I2C interface.
 * IMPORTANT: I2C line is LO.
 * [Slave addresses change according to the status of the I2C line (HI/LO)]
 *
 * TODO:
 *  Change the delay() function to udelay().
 *  Check if udelay can be used in kernel code.
 */

/*=========================================================
 This routine is used internally
 in:  bValue - value to be modified and returned
 return TRUE - successful
        FALSE - fail
 =========================================================*/
int readI2C(unsigned char* bValue)
{
  int nReturnValue = TRUE;
  int i;
  unsigned char uchValue = 0;
  unsigned char bBit = 0x80;  /* read from high bit */

  *bValue = 0;

  for (i=0; i<8; i++)
  {
    outb_p(0xb6, IGSEXTINDEX);
    outb_p(0x40, IGSEXTDATA);
    udelay(TIME1);
    outb_p(0x50, IGSEXTDATA);
    udelay(TIME1);

    if (0 != AcknowledgeClk())
      nReturnValue = FALSE;

    uchValue = inb_p(IGSEXTDATA);
    udelay(TIME1);

    if (0xF0 == uchValue)
    {
      *bValue = *bValue | bBit;
    }

    bBit = bBit >> 1;
  }

  return nReturnValue;
}

/**********************************************************
  Read one byte from I2C of the Decoder
  in:
    bAddr         = I2C slave address
    bSubAddr      = I2C subaddress
    bValue        = value to be returned
  return:
    TRUE  if successful
    FALSE if error
 *********************************************************/
int readOneByte(unsigned char bAddr,
                unsigned char bSubaddr,
                unsigned char* bValue)
{
  int nReturnValue = TRUE;

  if (FALSE == Start_Cycle())
    nReturnValue = FALSE;

  if (FALSE == writeI2C(bAddr & 0xFE))
    nReturnValue = FALSE;

  if (0 != Acknowledge())
    nReturnValue = FALSE;

  if (FALSE == writeI2C(bSubaddr))
    nReturnValue = FALSE;

  if (0 != Acknowledge())
    nReturnValue = FALSE;

  if (FALSE == Stop_Cycle())
    nReturnValue = FALSE;

  if (FALSE == Start_Cycle())
    nReturnValue = FALSE;

  if (FALSE == writeI2C(bAddr | 0x01))
    nReturnValue = FALSE;

  if (0 != Acknowledge())
    nReturnValue = FALSE;

  if (FALSE == readI2C(bValue))
    nReturnValue = FALSE;

  if (0 != AcknowledgeMaster())
    nReturnValue = FALSE;

  if (FALSE == Stop_Cycle())
    nReturnValue = FALSE;

  udelay(STOPTIME);

  return nReturnValue;
}

/*=========================================================
 This routine is used internally
 in:   bVal
 out:  return ture if successful
 =========================================================*/
int writeI2C(unsigned char bVal)
{
  int  i;
  unsigned char bBit = 0x80;  /* Start from high bit -- Bit 7. Write each bit.*/
  int bReturn = TRUE;

  for (i=0; i<8; i++)
  {
    if ( bVal & bBit )
    {
      outb_p(0xb6, IGSEXTINDEX);
      outb_p(0x40, IGSEXTDATA);
      udelay(TIME1);
      outb_p(0x50, IGSEXTDATA);
      udelay(TIME1);

      if (AcknowledgeClk()!=0)
        bReturn = FALSE;

      outb_p(0x40, IGSEXTDATA);
      udelay(TIME1);
    }
    else
    {
      outb_p(0xb6, IGSEXTINDEX);
      outb_p(0x00, IGSEXTDATA);
      udelay(TIME1);
      outb_p(0x10, IGSEXTDATA);
      udelay(TIME1);

      if (AcknowledgeClk()!=0)
        bReturn = FALSE;

      outb_p(0x00, IGSEXTDATA);
      udelay(TIME1);
    }
    bBit = bBit >> 1;
  }

  return bReturn;
}

/*=========================================================
 This routine is used internally
 in:   none
 return  0
 =========================================================*/
int AcknowledgeMaster(void)
{
  outb_p(0xb6, IGSEXTINDEX);
  outb_p(0x40, IGSEXTDATA);
  udelay(TIME1);
  outb_p(0x50, IGSEXTDATA);
  udelay(TIME1);

  outb_p(0x40, IGSEXTDATA);
  udelay(TIME1);
  outb_p(0x00, IGSEXTDATA);
  udelay(TIME1);

  return 0;
}

/*=========================================================
 This routine is used internally
 in:   none
 return  0 if successful
  -1 if error
 =========================================================*/
int Acknowledge(void)
{
  unsigned long i=0L;

  outb_p(0xb6, IGSEXTINDEX);
  outb_p(0x40, IGSEXTDATA);
  udelay(TIME1);
  outb_p(0x50, IGSEXTDATA);
  udelay(TIME1);
  while ( (inb_p(IGSEXTDATA) & 0x80) && (i++<TimeDelay1) );

  outb_p(0x40, IGSEXTDATA);
  udelay(TIME1);
  outb_p(0x00, IGSEXTDATA);
  udelay(TIME1);

  if (i< TimeDelay1)
    return 0;
  else
    return -1;
}

/*=========================================================
 This routine is used internally
 in:   none
 return  0 if successful
  -1 if error
 =========================================================*/
int AcknowledgeClk(void)
{
  unsigned long i=0;

  outb_p(0xb6, IGSEXTINDEX);
  while ( !(inb_p(IGSEXTDATA) & 0x20) && (i++<TimeDelay1) );

  if (i<TimeDelay1)
    return 0;
  else
    return -1;
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
int Start_Cycle(void)
{
  int bReturn = TRUE;

  outb_p(0xb6, IGSEXTINDEX);
  udelay(TIME1);
  outb_p(0x50, IGSEXTDATA);
  udelay(TIME1);
  outb_p(0x10, IGSEXTDATA);
  udelay(TIME1);

  if (AcknowledgeClk()!=0)
    bReturn = FALSE;

  udelay(TIME1);
  outb_p(0x00, IGSEXTDATA);
  udelay(TIME1);

  return bReturn;
}

/*=========================================================
 This routine is used internally
 in:
 =========================================================*/
int Stop_Cycle(void)
{
  int bReturn = TRUE;

  outb_p(0xb6, IGSEXTINDEX);
  udelay(TIME1);
  outb_p(0x10, IGSEXTDATA);
  udelay(TIME1);
  outb_p(0x50, IGSEXTDATA);
  udelay(TIME1);

  if (AcknowledgeClk()!=0)
    bReturn = FALSE;

  return bReturn;
}

/**********************************************************
Program one byte value to I2C of the Decoder
 in:   bAddr    = I2C port address
   bSubaddr = I2C subport address
   bVal   = date to be programmed
 return  TRUE  if successful
   FALSE if error
 *********************************************************/
int sendOneByte(unsigned char bAddr, unsigned char bSubaddr, unsigned char bVal){
  int bReturn = TRUE;

  if (Start_Cycle() == FALSE)
    bReturn = FALSE;

  if (writeI2C(bAddr) == FALSE)
    bReturn = FALSE;

  if (Acknowledge()!=0)
    bReturn = FALSE;

  if (writeI2C(bSubaddr) == FALSE)
    bReturn = FALSE;

  if (Acknowledge()!=0)
    bReturn = FALSE;

  if (writeI2C(bVal) == FALSE)
    bReturn = FALSE;

  if (Acknowledge()!=0)
    bReturn = FALSE;

  if (Stop_Cycle() == FALSE)
    bReturn = FALSE;

  udelay(STOPTIME);

  return bReturn;
}

/**********************************************************
Program more bytes to I2C of the Decoder
 in:   bAddr    = I2C port address
   bSubaddr = I2C subport address
   bValPtr  = pointer to data array
   iCount   = counter (# of bytes to be programmed)
 return  TRUE  if successful
   FALSE if error
 *********************************************************/
int sendMoreByte(unsigned char bAddr, unsigned char bSubaddr, unsigned char *bValPtr, int iCount)
{
  int bReturn = TRUE;
  unsigned char bVal;
  int i;

  if (Start_Cycle() == FALSE)
    bReturn = FALSE;

  if (writeI2C(bAddr) == FALSE)
    bReturn = FALSE;

  if (Acknowledge()!=0)
    bReturn = FALSE;

  if (writeI2C(bSubaddr) == FALSE)
    bReturn = FALSE;

  if (Acknowledge()!=0)
    bReturn = FALSE;

  for (i=0; i<iCount; i++)
  {
    bVal = *bValPtr++;
    if (writeI2C(bVal) == FALSE)
      bReturn = FALSE;

    if (Acknowledge()!=0)
      bReturn = FALSE;
  }

  if (Stop_Cycle() == FALSE)
    bReturn = FALSE;

  udelay(STOPTIME);

  return bReturn;
}

