#include "igs.h"
#include "igs_tbls.h"
#include "video.h"

int ii;

void set_resolution(int x_ext, int y_ext)
{
  /*-------------------------------------
  |  Step 2) Set mode										|
  -------------------------------------*/

  /*	The source code has already included following resolutions:
	-------------------------------------------
	| Resolution|  NTSC (60Hz) |  PAL (50Hz)	|
	|-----------|--------------|--------------|
	|  640x440  | CRT/TV (O/U) |     N/A      |
	|  640x480  | CRT/TV(O/U/I)|    CRT/TV    |
	|  720x540  |    N/A       |    CRT/TV    |
	|  800x600  |    CRT       |    CRT/TV    |
	| 1024x768  |    CRT       |     N/A      |
	-------------------------------------------
  Note: 1). CRT/TV: Support both Monitor and TV.
					. N/A:		Not support or not applicable.
					. CRT:		Support Monitor only, no TV.
					. O/U:		Support Overscan/Underscan mode.
					. O/U/I:  Support Overscan/Underscan/Interpolation mode.
	    	2). Please refer to tables in set2010.txt for
		   			more resolution, such as 1280x1024 or 1600x1200.
		   			more refresh rates, such as 56Hz, 60Hz, 72Hz or 75Hz.
	       		The extra tables in set2010.txt can be easily combined into
	       		IGS_TBLS.H to support desired modes.
	    	3)	Underscan:
	       		640x480: Underscan applies to both horizontal and vertical.
	       		640x440: Underscan applies to horizontal only. Vertical lines
						fit into screen by nature, so don't need underscan.
	    	4)	Interpolation: applies to vertical lines only.
	*/

	/* The x_ext/y_ext must be set to distinguish individual resolution */

	/* set to a specific mode */
	switch (y_ext)
	{
	  case 440:  /*640x440*/
	    SetStandardReg(S640x440x60);
	    SetPartExtReg(E640x440x8x60, EXTPARTIALCOUNT);
    break;

	  case 480:  /*640x480*/
		//  SetStandardReg(S640x480x60_U);
		//  SetPartExtReg(E640x480x8x60_U, EXTPARTIALCOUNT);

			SetStandardReg(S640x480x50);
			SetPartExtReg(E640x480x16x50, EXTPARTIALCOUNT);

		//	SetStandardReg(S640x440x60_U);
		//	SetPartExtReg(E640x440x8x60_U, EXTPARTIALCOUNT);
	  break;

	  case 540:  /*720x540*/
	    SetStandardReg(S720x540x50);
	    SetPartExtReg(E720x540x8x50, EXTPARTIALCOUNT);
	  break;

	  case 600:  /*800x600*/
			SetStandardReg(S800x600x75);
			SetPartExtReg(E800x600x16x75, EXTPARTIALCOUNT);
	  break;

	  case 768:  /*1024x768*/
	    SetStandardReg(S1024x768x60);
	    SetPartExtReg(E1024x768x8x60, EXTPARTIALCOUNT);
	  break;
	}

	BypassMode(ON);

	return;
}

/*--------------------------------------------------------------------
  SetStandardReg: Set standard VGA registers
  In:	bPtr - a BYTE pointer to an array of VGA register settings
  Out:	VGA registers are set
  --------------------------------------------------------------------*/
void SetStandardReg(unsigned char *bPtr)
{
 /* 1. set sequencer reg */
    outb_p(0x00, SEQINDEX);
    outb_p(0x00, SEQDATA);			/*seq clear*/

    for (ii=0; ii<SEQCOUNT; ii++)
    {
      outb_p(ii+1, SEQINDEX);		/*start from index 1*/
      outb_p(*bPtr++, SEQDATA);
    }

 /* 2. set miscellaneous reg */
    outb_p(*bPtr++, MISCWRITE);

    outb_p(0x00, SEQINDEX);
    outb_p(0x03, SEQDATA);			/*seq normal operation*/

 /* 3. set CRT reg */
    outb_p(0x11, CRTINDEX);
    outb_p(0x00, CRTDATA);			/*unlock CRT*/

    for (ii=0; ii<CRTCOUNT; ii++)
    {
      outb_p(ii, CRTINDEX);
      outb_p(*bPtr++, CRTDATA);
    }

 /* 4. set attribute reg */
    inb_p(ATTRRESET);						/*reset Attribute*/

    for (ii=0; ii<ATTRCOUNT; ii++)
    {
      outb_p(ii, ATTRINDEX);
      outb_p(*bPtr++, ATTRDATAW);
    }

    outb_p(ii, ATTRINDEX);			/*normal case*/
    outb_p(0x00, ATTRDATAW);		/*3c0_reg_14 is always 0*/

    outb_p(0x20, ATTRINDEX);		/*turn on screen*/
    inb_p(ATTRRESET);						/*reset Attribute*/

 /* 5. set graphics reg */
    for (ii=0; ii<GRACOUNT; ii++)
    {
      outb_p(ii, GRAINDEX);
      outb_p(*bPtr++, GRADATA);
    }
  return;
}

/*--------------------------------------------------------------------
  SetExtReg: Set IGS partial extended registers
  In:	bPtr - a BYTE pointer to an array of IGS extended register settings
	       and VGA CRT register 0x13.
	iCounter - a counter of the extended registers only.
  Out:	None
  Note: After initialization, setmode only requires to set VGA registers and
	partial IGS extended registers. Most of the IGS extended registers
	remain no change.
  --------------------------------------------------------------------*/
void SetPartExtReg(unsigned char *bPtr, int iCounter)
{
	outb_p(0x13, CRTINDEX);		 /*index*/
	outb_p(*bPtr++, CRTDATA);	 /*data*/

	for (ii=0; ii<iCounter; ii++)
	{
		outb_p(*bPtr++, EXTINDEX); /*index*/
		outb_p(*bPtr++, EXTDATA);	 /*data*/
	}
	ToggleClock();
}

void ToggleClock(void)
{
  unsigned char bTmp;

  outb_p(0xB9, EXTINDEX);	/*bit <7> is the control bit*/
  bTmp = inb_p(EXTDATA);

  outb_p(bTmp & 0x7F, EXTDATA);
  outb_p(bTmp | 0x80, EXTDATA);
  outb_p(bTmp & 0x7F, EXTDATA);
}

void EnableChip1()
{
  outb_p(0x18, PORT46E8);     /*46E8 is writeable only*/
  outb_p(0x01, PORT102);     /*102 is read-writeable*/
  outb_p(0x08, PORT46E8);
}

void InitChip1()
{
  SetExtReg(ExtRegData, sizeof(ExtRegData)/2);
  SetStandardReg(S640x480x60);
 //   SetRamDac(RamDacData, sizeof(RamDacData)/3);

  /*choose voltage reference*/
  outb_p(0x56, EXTINDEX);
  outb_p(0x04, EXTDATA);
  outb_p(0x04, RAMDACMASK);           /*use VRef*/
  outb_p(0,    EXTDATA);
  outb_p(0x56, EXTINDEX);
  outb_p(0x00, EXTDATA);
}

void SetExtReg(unsigned char *bPtr, int iCounter)
{
  for (ii=0; ii<iCounter; ii++)
  {
    outb_p(*bPtr++, EXTINDEX);       /*index*/
    outb_p(*bPtr++, EXTDATA);       /*data*/
  }

  ToggleClock();        /*to latch the clock value into working set*/
}

void SetRamDac(unsigned char *bPtr, int iCounter)
{
  outb_p(0, RAMDACINDEXW);           /*start from index 0*/
  for (ii=0; ii<iCounter; ii++)
  {
    outb_p(*bPtr++, RAMDACDATA);     /*R*/
    outb_p(*bPtr++, RAMDACDATA);     /*G*/
    outb_p(*bPtr++, RAMDACDATA);     /*B*/
  }

  outb_p(0xff, RAMDACMASK);          /*Mask register*/
}

void BypassMode(int iOnOff)
{
	BYTE bTmp;

	outb_p(0x56, EXTINDEX);
	bTmp = inb_p(EXTDATA);
	outb_p(bTmp | 0x04, EXTDATA);

	if (ON == iOnOff)
		outb_p(inb_p(RAMDACMASK) | 0x10, RAMDACMASK);
	else
		outb_p(inb_p(RAMDACMASK) & ~0x10, RAMDACMASK);	

	outb_p(bTmp & ~0x04, EXTDATA);
}
